'use client'

import { Cookies, OrientationMapper, Platforms } from '@/meta/settings';

import css from './Platform.module.scss'

import React from 'react';

import { setCookie } from '@/utils/cookies/write';
import { useCookies } from 'react-cookie';

interface Props {
    handleClose(): void
    enabledPlatforms: string[]
    labels: {
        title: string,
    }
}

const mapOrientationToPlatform = (platform: string, orientation: string): string | undefined => {
    const orientations = OrientationMapper[platform as Platforms]
    return Object.keys(orientations).includes(orientation) ? orientation : String(Object.keys(orientations)[0])
}

const Platform: React.FC<Props> = (props) => {

    const { labels, handleClose, enabledPlatforms } = props

    const [cookies] = useCookies([Cookies.platform, Cookies.orientation]);

    const handleChange = async (event: React.ChangeEvent<HTMLSelectElement>) => {
        const value = event.target.value;

        await setCookie(Cookies.platform, value)

        if (cookies.orientation) {
            const newOrientation = mapOrientationToPlatform(value, cookies.orientation)
            newOrientation && await setCookie(Cookies.orientation, newOrientation)
        }

        handleClose()
    }

    return (
        <div className={css.container}>
            <div className={css.title}>{labels.title}</div>
            <select defaultValue={cookies.platform ?? Platforms.xvideos} onChange={handleChange} name={'platform'} aria-label={labels.title}>
                {Object.keys(Platforms).map((elem, key) => {
                    if (!enabledPlatforms.includes(elem)) {
                        return null
                    }

                    return <option className={css.option} key={key} value={elem}>{elem.toUpperCase()}</option>
                })}
            </select>
        </div>
    );
};

export default Platform;