import { XHAMSTER_BASE_URL, XHAMSTER_BASE_URL_VIDEOS } from "@/constants/urls";
import { FetchParams, GalleryData } from "@/meta/data";
import { getHeaders } from "../common/headers";
import { getXHamsterQueryUrl } from "./url";
import { getDataFromRedis, storeDataIntoRedis } from "@/redis/client";

import * as cheerio from "cheerio";

import axios, { AxiosError } from "axios";
import { DEFAULT_XHAMSTER_GALLERY_EXPIRY } from "@/constants/redis";
import { Platforms } from "@/meta/settings";

export const fetchXHamsterGalleryData = async (params?: FetchParams): Promise<GalleryData[]> => {

    let data: GalleryData[] = [];

    const reqHeaders = getHeaders(XHAMSTER_BASE_URL)

    const queryUrl = await getXHamsterQueryUrl(params?.query)

    const cachedData = await getDataFromRedis(queryUrl)

    if (cachedData) {
        return cachedData as GalleryData[]
    }

    await axios.get(queryUrl, reqHeaders)

    .then(async response => {

        const html = response.data;

        const $ = cheerio.load(html);

        const wrapperId = '.thumb-list .thumb-list__item'
    
        const thumbs = $(wrapperId);
    
        thumbs.map((key, thumb) => {
    
            const videoUrl = $(thumb).find("a.video-thumb__image-container").attr("href")?.replace(XHAMSTER_BASE_URL_VIDEOS, '')
            const imgUrl = $(thumb).find("a.video-thumb__image-container img").attr("src")
            const text = $(thumb).find("a.video-thumb-info__name").attr("title")
    
            videoUrl && imgUrl && text && data.push({
                videoUrl,
                imgUrl,
                text,
                platform: Platforms.xhamster
            })
        })

        if (data.length > 0) {
            await storeDataIntoRedis(queryUrl, data, DEFAULT_XHAMSTER_GALLERY_EXPIRY);
        }

    }).catch((error: AxiosError) => {
        // handle errors
    });

    return data
}